/*
 * Copyright  2008 Nokia Corporation.
 */

#ifndef DRAWINGENGINE_H
#define DRAWINGENGINE_H

// INCLUDES
#include <e32std.h>
#include <e32base.h>
#include <gdi.h>
#include <w32std.h>

class CFbsBitGc;
class CToolbarControl;

// CLASS DECLARATION
class CDrawingEngine : public CBase
    {
    public:
        virtual ~CDrawingEngine ( );
        static CDrawingEngine* NewL (RWindow& aWindow, CWindowGc& aGc, CFbsBitGc* aBgc, CToolbarControl* aToolbarControl);
        static CDrawingEngine* NewLC (RWindow& aWindow, CWindowGc& aGc, CFbsBitGc* aBgc, CToolbarControl* aToolbarControl);

    public:
        void SetUseBgc(TBool aUseBgc); // CFbsBitGc or CWindowGc in use
        void SetColor(TRgb aColor);
        inline TRgb Color(){return iColor;};
        void SetPenSize(TSize aSize);
        void DoDrawing(const TPointerEvent& aPointerEvent);
        void SetNewBgc(CFbsBitGc* aBgc);
        void ResetDrawing();

    private:
        CDrawingEngine (RWindow& aWindow, CWindowGc& aGc, CFbsBitGc* aBgc, CToolbarControl* aToolbarControl);
        void ConstructL ( );

        void DoFreeDraw(const TPointerEvent& aPointerEvent);
        void DoCircle(const TPointerEvent& aPointerEvent);
        void DoLine(const TPointerEvent& aPointerEvent);
        void DoRect(const TPointerEvent& aPointerEvent);

        TBool LineReady();
        TBool CircleReady();
        TBool RectangleReady();

    private:
        RWindow&            iWindow; 
        CWindowGc&          iGc;
        CFbsBitGc*          iBgc;
        TBool               iUseBgc;
        
        CToolbarControl*    iToolbarControl;
        TRgb                iColor;
        TSize               iSize;
        
        TPoint              iLinePoint1;
        TPoint              iLinePoint2;
        TPoint              iCircleCenter;
    
    };

#endif // DRAWINGENGINE_H
