/*
 * Copyright  2008 Nokia Corporation.
 */

// INCLUDE FILES
#include <coemain.h>
#include "MobilePaintAppView.h"
#include "cdrawingengine.h"
#include "ctoolbarcontrol.h"
#include "mobilepaintappui.h"
#include "csvghandler.h"
#include "MobilePaint.hrh"
#include <aknutils.h>
#include <akntoolbar.h>
#include <akntoolbarextension.h>
#include <MobilePaint_0xA000A332.rsg>

// ============================ MEMBER FUNCTIONS ===============================


CBitmapRotatorAO::CBitmapRotatorAO(MBitmapRotatorObserver* aObserver)
: CActive(EPriorityStandard), iObserver(aObserver)
    {
    }

CBitmapRotatorAO* CBitmapRotatorAO::NewL(MBitmapRotatorObserver* aObserver)
    {
    CBitmapRotatorAO* self = CBitmapRotatorAO::NewLC(aObserver );
    CleanupStack::Pop (self );
    return self;
    }

CBitmapRotatorAO* CBitmapRotatorAO::NewLC(MBitmapRotatorObserver* aObserver)
    {
    CBitmapRotatorAO* self = new ( ELeave ) CBitmapRotatorAO(aObserver);
    CleanupStack::PushL (self );
    self->ConstructL();
    return self;
    }

CBitmapRotatorAO::~CBitmapRotatorAO()
    {
    Cancel();
    delete iBitmapRotator;
    }

void CBitmapRotatorAO::ConstructL()
    {
    CActiveScheduler::Add(this);
    iBitmapRotator = CBitmapRotator::NewL();
    }

void CBitmapRotatorAO::DoCancel()
    {
    iBitmapRotator->Cancel();
    }

void CBitmapRotatorAO::RunL()
    {
    iObserver->RotationCompletedL(iStatus.Int());
    }

TInt CBitmapRotatorAO::RunError(TInt aError)
    {
    TRAP_IGNORE(iObserver->RotationCompletedL(aError));
    return KErrNone;
    }

void CBitmapRotatorAO::Rotate(CBitmapRotator::TRotationAngle aAngle, CFbsBitmap* aTarget)
    {
    if (!IsActive())
        {
        iBitmapRotator->Rotate(&iStatus,*aTarget,aAngle);
        SetActive();
        }
    else
        {
        TRAP_IGNORE(iObserver->RotationCompletedL(KErrNotReady));
        }
    }




// -----------------------------------------------------------------------------
// CMobilePaintAppView::NewL()
// Two-phased constructor.
// -----------------------------------------------------------------------------
//
CMobilePaintAppView* CMobilePaintAppView::NewL (const TRect& aRect )
    {
    CMobilePaintAppView* self = CMobilePaintAppView::NewLC (aRect );
    CleanupStack::Pop (self );
    return self;
    }

// -----------------------------------------------------------------------------
// CMobilePaintAppView::NewLC()
// Two-phased constructor.
// -----------------------------------------------------------------------------
//
CMobilePaintAppView* CMobilePaintAppView::NewLC (const TRect& aRect )
    {
    CMobilePaintAppView* self = new ( ELeave ) CMobilePaintAppView;
    CleanupStack::PushL (self );
    self->ConstructL (aRect );
    return self;
    }

// -----------------------------------------------------------------------------
// CMobilePaintAppView::ConstructL()
// Symbian 2nd phase constructor can leave.
// -----------------------------------------------------------------------------
//
void CMobilePaintAppView::ConstructL (const TRect& /*aRect*/ )
    {
    // Create a window for this application view
    CreateWindowL ( );

    EnableDragEvents();

    // This class rotates backbuffer (=picture on screen) when terminal
    // oreintation changes
    iBackBufferRotator = CBitmapRotatorAO::NewL(this);

    iAppUi = (static_cast<CMobilePaintAppUi*>(iEikonEnv->AppUi()));
    iAknToolbar = iAppUi->CurrentPopupToolbar();

    // CAknToolbar control
    #ifdef ENABLE_AVKON_TOOLBAR
    iAknToolbar->SetToolbarVisibility( ETrue, EFalse );
    iAknToolbar->SetToolbarObserver( this );
    CAknButton* button = static_cast<CAknButton*>(iAknToolbar->ControlOrNull(EToolBarSize));
    button->SetCurrentState(2, EFalse); 
    #endif

    // Use whole screen
    SetExtentToWholeScreen();
    
    // Initialize component array
    InitComponentArrayL();
    
    // Custom CToolbarControl toolbar control
    iCustomToolbarControl = CToolbarControl::NewL(this,Rect());
    // If CAknToolbar is used, then hide custom toolbar
    #ifdef ENABLE_AVKON_TOOLBAR
    iCustomToolbarControl->MakeVisible(EFalse);
    #endif
    iCustomToolbarControl->SetLeft();
    Components().AppendLC(iCustomToolbarControl);
    CleanupStack::Pop(); // iCustomToolbarControl

    // Create drawing engine
    iDrawingEngine = CDrawingEngine::NewL(Window(),SystemGc(),iBackBufferContext,iCustomToolbarControl);
    iDrawingEngine->SetUseBgc(ETrue); // Draws default to back buffer
    iDrawingEngine->SetColor(KRgbRed); // Use read color as default
    iDrawingEngine->SetPenSize(TSize(KMINPenSize,KMINPenSize)); // Default pen size
    
    // Activate the window, which makes it ready to be drawn
    ActivateL ( );
    }

// -----------------------------------------------------------------------------
// CMobilePaintAppView::CMobilePaintAppView()
// C++ default constructor can NOT contain any code, that might leave.
// -----------------------------------------------------------------------------
//
CMobilePaintAppView::CMobilePaintAppView ( )
    {
    // No implementation required
    }

// -----------------------------------------------------------------------------
// CMobilePaintAppView::~CMobilePaintAppView()
// Destructor.
// -----------------------------------------------------------------------------
//
CMobilePaintAppView::~CMobilePaintAppView ( )
    {
    ReleaseBackBuffer();
    delete iDrawingEngine;
    delete iBackBufferRotator;
    delete iCopyOfBackBuffer;
    delete iCopyBackBufferDevice;
    }

// -----------------------------------------------------------------------------
// CMobilePaintAppView::Draw()
// Draws the display.
// -----------------------------------------------------------------------------
//
void CMobilePaintAppView::Draw (const TRect& /*aRect*/) const
    {
    // Get the standard graphics context
    CWindowGc& gc = SystemGc ( );

    if (iBitmapRotationRunning)
        {
        return;
        }
    
    // Copy backbuffer into the screen
    // Backbuffer has all drawed shapes
    gc.BitBlt(TPoint(0, 0), iBackBuffer);
    
    // All new drawing goes to CWindowGc and after pointer up
    // it goes into backbuffer.
    // Do draving with tool NOTE: into CWindowGc
    if (iControlState == EIdle)
        {
        // User was drawing with stylus
        if (iAcceptDrawingRect.Contains(iPointerEvent.iPosition))
            {
            iDrawingEngine->DoDrawing(iPointerEvent);
            }
        else
            {
            // Reset drawing when stylus goes out of screen (accepted rect)
            iDrawingEngine->ResetDrawing();
            }
        }

    // Draw CBA buttons
    DrawCbaButtons(gc);
    }


void CMobilePaintAppView::DrawCbaButtons(CWindowGc& aGc) const
    {
    // Draw CBA background
    if (iAppUi->CbaPosition() == AknLayoutUtils::EAknCbaLocationBottom)
        {
        // Terminal portrait
        TRect background = TRect(TPoint(0,0),TSize(KCBAPortraitWidth,KCBAPortraitHeight));
        aGc.BitBltMasked(iCbaBackground.iTl, iAppUi->SvgHandler()->iCBAPortraitBitmap, 
                background, iAppUi->SvgHandler()->iCBAPortraitBitmapMask, EFalse);
        }
    else
        {
        // Terminal landscape
        TRect background = TRect(TPoint(0,0),TSize(KCBALandscapeWidth,KCBALandscapeHeight));
        aGc.BitBltMasked(iCbaBackground.iTl, iAppUi->SvgHandler()->iCBALandscapeBitmap, 
                background, iAppUi->SvgHandler()->iCBALandscapeBitmapMask, EFalse);
        }
    
    // Draw CBA buttons
    TRect buttonRect = TRect(TPoint(0,0),TSize(KToolbarItemWidth,KToolbarItemHeight));
    aGc.BitBltMasked(iOptionCBA.iTl, iAppUi->SvgHandler()->iMenuButtonBitmap, 
            buttonRect, iAppUi->SvgHandler()->iMenuButtonBitmapMask, EFalse);
    aGc.BitBltMasked(iExitCBA.iTl, iAppUi->SvgHandler()->iExitButtonBitmap, 
            buttonRect, iAppUi->SvgHandler()->iExitButtonBitmapMask, EFalse);
    }

void CMobilePaintAppView::NewPictureL()
    {
    if (iEikonEnv->QueryWinL(_L("Mobile Paint"),_L("Clear screen?")))
        {
        ClearBackground();
        }
    }
// -----------------------------------------------------------------------------
// CMobilePaintAppView::SizeChanged()
// Called by framework when the view size is changed.
// -----------------------------------------------------------------------------
//
void CMobilePaintAppView::SizeChanged ( )
    {
    // Create acceptable drawing area
    iAcceptDrawingRect = Rect();
    iAcceptDrawingRect.Shrink(5,5);
    
    // Create back buffer. If orientation has chanded it rotates
    // current backbuffer
    TRAP_IGNORE(CreateBackBufferL());


    // CAknToolbar position
    #ifdef ENABLE_AVKON_TOOLBAR
    if (iAknToolbar)
        {
        if (iAppUi->CbaPosition() == AknLayoutUtils::EAknCbaLocationRight)
            {
            // Landscape
            iAknToolbar->SetOrientation( EAknOrientationHorizontal );
            }
        else
            {
            // Portrait
            iAknToolbar->SetOrientation( EAknOrientationVertical );
            }
        }
    #else
    // Custom CToolbarControl position
    if (iCustomToolbarControl && iCustomToolbarControl->IsVisible())
        {
        if (iDrawingEngine && iCustomToolbarControl)
            {
            if (iAppUi->CbaPosition() == AknLayoutUtils::EAknCbaLocationBottom)
                {
                iCustomToolbarControl->SetLeft();
                }
            else
                {
                iCustomToolbarControl->SetTop();
                }
            }
        }
    #endif
    
    // Calculate CBA buttons positions
    if (iAppUi->CbaPosition() == AknLayoutUtils::EAknCbaLocationBottom)
        {
        // CBA buttons bottom (portrait orientation)
        iOptionCBA = TRect(
                TPoint(KToolbarGap,Size().iHeight - KToolbarGap - KToolbarItemHeight),
                TSize(KToolbarItemWidth,KToolbarItemHeight));
        iExitCBA = TRect(
                TPoint(Size().iWidth - KToolbarItemWidth - KToolbarGap,Size().iHeight - KToolbarGap - KToolbarItemHeight),
                TSize(KToolbarItemWidth,KToolbarItemHeight));

        iCbaBackground = TRect(
                TPoint(0,Size().iHeight - KCBAPortraitHeight),
                TSize(KCBAPortraitWidth,KCBAPortraitHeight));
        }
    else
        {
        // CBA buttons left (landscape orientation)
        iExitCBA = TRect(
                TPoint(Size().iWidth - KToolbarItemWidth - KToolbarGap,KToolbarGap),
                TSize(KToolbarItemWidth,KToolbarItemHeight));

        iOptionCBA = TRect(
                TPoint(Size().iWidth - KToolbarItemWidth - KToolbarGap,Size().iHeight - KToolbarGap - KToolbarItemHeight),
                TSize(KToolbarItemWidth,KToolbarItemHeight));
        
        iCbaBackground = TRect(
                TPoint(Size().iWidth - KCBALandscapeWidth,0),
                TSize(KCBALandscapeWidth,KCBALandscapeHeight));
        }
    }


void CMobilePaintAppView::HandleResourceChange(TInt aType)
    {
    CCoeControl::HandleResourceChange(aType);
    if ( aType==KEikDynamicLayoutVariantSwitch )
        {
        SetExtentToWholeScreen();        
        }
    }

void CMobilePaintAppView::HandlePointerEventL(const TPointerEvent& aPointerEvent)
    {
    iPointerEvent = aPointerEvent;
    
    // Decide used graphic context for drawing
    iDrawingEngine->SetUseBgc(ETrue);
    if ( iCustomToolbarControl->ActiveTool() == ECircle ||
         iCustomToolbarControl->ActiveTool() == ELine ||
         iCustomToolbarControl->ActiveTool() == ERect
       )
        {
        if (aPointerEvent.iType == TPointerEvent::EDrag)
            {
            // All drawing with stylus goes into CWindowGc
            iDrawingEngine->SetUseBgc(EFalse);
            }
        }

    // Moving toolbar state
    if (aPointerEvent.iType == TPointerEvent::EButton1Down)
        {
        if (iCustomToolbarControl->Rect().Contains(aPointerEvent.iPosition))
            {
            // User is moving toolbar -> no drawing with tool now
            iControlState = EPointerOnToolbar;
            }
        }

    // Hitting CBA
    if (aPointerEvent.iType == TPointerEvent::EButton1Down)
        {
        if (iExitCBA.Contains(aPointerEvent.iPosition))
            {
            iAppUi->HandleCommandL(EEikCmdExit);
            return;
            }
        else if (iOptionCBA.Contains(aPointerEvent.iPosition))
            {
            iEikonEnv->AppUiFactory()->MenuBar()->TryDisplayMenuBarL();
            return;
            }
        }

    // For UNDO functionality
    // Copy current backbuffer (=screen image) into memory when user
    // starts new drawing.
    // If user wants later undo drawing, we have this old screen image in memory
    if (aPointerEvent.iType == TPointerEvent::EButton1Down)
        {
        #ifdef ENABLE_AVKON_TOOLBAR
        if (!iAknToolbar->Rect().Contains(aPointerEvent.iPosition))
            {
            // Copy back buffer if user is starting to do some new drawing
            CopyBackBufferL();
            }
        #else
        if (!iCustomToolbarControl->Rect().Contains(aPointerEvent.iPosition))
            {
            // Copy back buffer if user is starting to do some new drawing
            CopyBackBufferL();
            }
        #endif        
        }
    
    
    // Call all child ui controls
    // (Sends pointer events to custom toolbar)
    CCoeControl::HandlePointerEventL(aPointerEvent);

    // Do all drawing, that calls Draw() function
    // NOTE: iDrawingEngine->DoDrawing(iPointerEvent) is called in Draw() function
    DrawNow();
    
    // Draw toolbar last, then it is top of all others
    if (iCustomToolbarControl && iCustomToolbarControl->IsVisible())
        {
        iCustomToolbarControl->DrawNow();
        }
    
    // Invalid move toolbar state
    if (aPointerEvent.iType == TPointerEvent::EButton1Up)
        {
        iControlState = EIdle;
        }
    }

void CMobilePaintAppView::CopyBackBufferL()
    {
    // This copies current screen image into memory for undo support
    // Delete current copy
    delete iCopyOfBackBuffer;
    iCopyOfBackBuffer = NULL;
    iCopyOfBackBuffer = new (ELeave) CFbsBitmap;
    User::LeaveIfError( iCopyOfBackBuffer->Create(iBackBuffer->SizeInPixels(),iBackBuffer->DisplayMode()));
    
    // Create copy
    TSize size = iBackBuffer->SizeInPixels();
    iCopyOfBackBuffer->Create(size,iBackBuffer->DisplayMode());
    int scanLineLength = CFbsBitmap::ScanLineLength(size.iWidth, iBackBuffer->DisplayMode());
    if (iCopyOfBackBuffer && iCopyOfBackBuffer->DataAddress())
        {
        memcpy( (void*)iCopyOfBackBuffer->DataAddress(), (void*)iBackBuffer->DataAddress(), scanLineLength * size.iHeight );
        }    
    }

void CMobilePaintAppView::RestoreBackBufferL()
    {
    // We take old screen image from memory when user want to undo current drawing
    if (iCopyOfBackBuffer)
        {
        ReleaseBackBuffer();

        iBackBuffer = new (ELeave) CFbsBitmap;
        User::LeaveIfError( iBackBuffer->Create(iCopyOfBackBuffer->SizeInPixels(),iCopyOfBackBuffer->DisplayMode()));
        
        
        // Copy copied bitmap back to use
        TSize size = iCopyOfBackBuffer->SizeInPixels();
        iBackBuffer->Create(size,iCopyOfBackBuffer->DisplayMode());
        int scanLineLength = CFbsBitmap::ScanLineLength(size.iWidth, iCopyOfBackBuffer->DisplayMode());
        if (iBackBuffer && iBackBuffer->DataAddress())
            {
            memcpy( (void*)iBackBuffer->DataAddress(), (void*)iCopyOfBackBuffer->DataAddress(), scanLineLength * size.iHeight );
            }    
        
        iBackBufferDevice = CFbsBitmapDevice::NewL(iBackBuffer);
        User::LeaveIfError(iBackBufferDevice->CreateContext(iBackBufferContext));
        iBackBufferContext->SetPenStyle(CGraphicsContext::ESolidPen);
        iBackBufferSize = iBackBuffer->SizeInPixels();

        // Update drawing engine
        if (iDrawingEngine)
            {
            iDrawingEngine->SetNewBgc(iBackBufferContext);
            }
        }
    }


void CMobilePaintAppView::CreateBackBufferL()
    {
    // This back buffer is like paper where all user drawing goes
    // Back buffer is created only once
    if (!iBackBufferContext)
        {
        // create back buffer bitmap
        iBackBuffer = new (ELeave) CFbsBitmap;
        
        User::LeaveIfError( iBackBuffer->Create(Size(),EColor64K)); // 16 bpp
        
        // create back buffer graphics context
        iBackBufferDevice = CFbsBitmapDevice::NewL(iBackBuffer);
        User::LeaveIfError(iBackBufferDevice->CreateContext(iBackBufferContext));
        iBackBufferContext->SetPenStyle(CGraphicsContext::ESolidPen);
        iBackBufferSize = iBackBuffer->SizeInPixels();
        
        if (iDrawingEngine)
            {
            iDrawingEngine->SetNewBgc(iBackBufferContext);
            }
        }
    // Check that is there need to resize current backbuffer
    else
        {
        if (iBackBufferSize != Size())
            {
            // Resize current back buffer
            RotateBackBufferL();
            }
        }
    }

TInt CMobilePaintAppView::DataStride(const CFbsBitmap* aBitmap)
    {
    return CFbsBitmap::ScanLineLength( aBitmap->SizeInPixels().iWidth, aBitmap->DisplayMode() );
    }


void CMobilePaintAppView::RotateBackBufferL()
    {
    // Terminal screen orientation was changed, so time to rotate image
    if (Size().iWidth < Size().iHeight)
        {
        iBitmapRotationRunning = ETrue;
        iBackBufferRotator->Rotate(CBitmapRotator::ERotation90DegreesClockwise,iBackBuffer);
        }
    else
        {
        iBitmapRotationRunning = ETrue;
        iBackBufferRotator->Rotate(CBitmapRotator::ERotation270DegreesClockwise,iBackBuffer);
        }
    }
    
void CMobilePaintAppView::RotationCompletedL(TInt aError)
    {
    if (aError == KErrNone)
        {
        // Use new resized back buffer
        delete iBackBufferDevice;
        iBackBufferDevice = NULL;
        iBackBufferDevice = CFbsBitmapDevice::NewL(iBackBuffer);

        delete iBackBufferContext;
        iBackBufferContext = NULL;
        User::LeaveIfError(iBackBufferDevice->CreateContext(iBackBufferContext));
        iBackBufferContext->SetPenStyle(CGraphicsContext::ESolidPen);

        // Read new size
        iBackBufferSize = iBackBuffer->SizeInPixels();

        // Set new back buffer into iDrawingEngine
        if (iDrawingEngine)
            {
            iDrawingEngine->SetNewBgc(iBackBufferContext);
            }
        }
    
    // Copy new rotated buffer for undo
    CopyBackBufferL();
    
    iBitmapRotationRunning = EFalse;
    DrawNow();
    }


void CMobilePaintAppView::ReleaseBackBuffer()
    {
    if (iBackBufferContext)
        {
        delete iBackBufferContext;
        iBackBufferContext = NULL;
        }
    if (iBackBufferDevice)
        {
        delete iBackBufferDevice;
        iBackBufferDevice = NULL;
        }
    if (iBackBuffer)
        {
        delete iBackBuffer;
        iBackBuffer = NULL;
        }
    iBackBufferSize = TSize(0,0);
    }

void CMobilePaintAppView::ClearBackground()
    {
    // Clear current back buffer, user can start new drawing with empty screen
    if (!iBackBufferContext)
        {
        return;
        }
    iBackBufferContext->SetBrushColor(KRgbWhite);
    iBackBufferContext->Clear();
    }

void CMobilePaintAppView::OfferToolbarEventL(TInt aCommand)
    {
    switch (aCommand)
        {
        case EToolbarNew:
            {
            NewPictureL();
            break;
            }
        case EToolbarColor:
            {
            iCustomToolbarControl->SelectColourL();
            break;
            }
        case EToolbarUndo:
            {
            RestoreBackBufferL();
            break;
            }
        case EToolbarPencil:
            {
            iCustomToolbarControl->SetActiveTool(EPencil);
            iAknToolbar->ToolbarExtension()->SetShown(EFalse);
            break;
            }
        case EToolbarLine:
            {
            iCustomToolbarControl->SetActiveTool(ELine);
            iAknToolbar->ToolbarExtension()->SetShown(EFalse);
            break;
            }
        case EToolbarCircle:
            {
            iCustomToolbarControl->SetActiveTool(ECircle);
            iAknToolbar->ToolbarExtension()->SetShown(EFalse);
            break;
            }
        case EToolbarRectangle:
            {
            iCustomToolbarControl->SetActiveTool(ERect);
            iAknToolbar->ToolbarExtension()->SetShown(EFalse);
            }
        case EToolBarSize:
            {
            // Update button face
            CAknButton* button = static_cast<CAknButton*>(iAknToolbar->ControlOrNull(EToolBarSize));
            if (button->StateIndex() == 2) 
                { 
                button->SetCurrentState( 0, ETrue ); 
                iDrawingEngine->SetPenSize(TSize(KMINPenSize+4,KMINPenSize+4));
                }               
            else if (button->StateIndex() == 0) 
                { 
                button->SetCurrentState( 1, ETrue ); 
                iDrawingEngine->SetPenSize(TSize(KMINPenSize+2,KMINPenSize+2));
                }               
            else if (button->StateIndex() == 1) 
                { 
                button->SetCurrentState( 2, ETrue ); 
                iDrawingEngine->SetPenSize(TSize(KMINPenSize,KMINPenSize));
                }               
            break;
            }
        default:
            {
            break;
            }
        };
    DrawNow();
    }


// End of File
